@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 from shlex import shlex
 from types import GeneratorType
 import gettext
@@ -7,12 +7,11 @@ import locale
 import decimal
 from decimal import Decimal
 import datetime
-import time
 import io
 from collections import OrderedDict
 
-from tryton.translate import date_format
-from tryton.common import untimezoned_date, datetime_strftime
+from tryton.common import untimezoned_date, timezoned_date, datetime_strftime
+from tryton.common.datetime_ import date_parse
 from tryton.pyson import PYSONDecoder
 
 __all__ = ['DomainParser']
@@ -117,6 +116,8 @@ def quote(value):
     "Quote string if needed"
     if not isinstance(value, basestring):
         return value
+    if '\\' in value:
+        value = value.replace('\\', '\\\\')
     if '"' in value:
         value = value.replace('"', '\\"')
     for test in (':', ' ', '(', ')') + OPERATORS:
@@ -129,6 +130,7 @@ def test_quote():
     assert quote('test') == 'test'
     assert quote('foo bar') == '"foo bar"'
     assert quote('"foo"') == '\\\"foo\\\"'
+    assert quote('foo\\bar') == 'foo\\\\bar'
 
 
 def ending_clause(domain, deep=0):
@@ -220,8 +222,10 @@ def test_split_target_value():
         assert split_target_value(field, value) == result
 
 
-def convert_value(field, value):
+def convert_value(field, value, context=None):
     "Convert value for field"
+    if context is None:
+        context = {}
 
     def convert_boolean():
         if isinstance(value, basestring):
@@ -256,28 +260,31 @@ def convert_value(field, value):
         return value
 
     def convert_datetime():
+        if not value:
+            return
+        format_ = context.get('date_format', '%X') + ' %x'
         try:
-            return untimezoned_date(datetime.datetime(*time.strptime(value,
-                        date_format() + ' ' + time_format(field))[:6]))
+            dt = date_parse(value, format_)
+            if dt.time() == datetime.time.min:
+                return dt
+            return untimezoned_date(dt)
         except ValueError:
-            try:
-                return datetime.datetime(*time.strptime(value,
-                        date_format())[:6])
-            except ValueError:
-                return
-        except TypeError:
             return
 
     def convert_date():
+        if not value:
+            return
+        format_ = context.get('date_format', '%x')
         try:
-            return datetime.date(*time.strptime(value, date_format())[:3])
+            return date_parse(value, format_).date()
         except (ValueError, TypeError):
             return
 
     def convert_time():
+        if not value:
+            return
         try:
-            return datetime.time(*time.strptime(value,
-                    time_format(field))[3:6])
+            return date_parse(value).time()
         except (ValueError, TypeError):
             return
 
@@ -423,8 +430,10 @@ def test_convert_time():
         assert convert_value(field, value) == result
 
 
-def format_value(field, value, target=None):
+def format_value(field, value, target=None, context=None):
     "Format value for field"
+    if context is None:
+        context = {}
 
     def format_boolean():
         return _('True') if value else _('False')
@@ -458,16 +467,20 @@ def format_value(field, value, target=None):
     def format_datetime():
         if not value:
             return ''
-        format_ = date_format() + ' ' + time_format(field)
+        format_ = context.get('date_format', '%x') + ' ' + time_format(field)
         if (not isinstance(value, datetime.datetime)
                 or value.time() == datetime.time.min):
-            format_ = date_format()
-        return datetime_strftime(value, format_)
+            format_ = '%x'
+            time = value
+        else:
+            time = timezoned_date(value)
+        return datetime_strftime(time, format_)
 
     def format_date():
         if not value:
             return ''
-        return datetime_strftime(value, date_format())
+        format_ = context.get('date_format', '%x')
+        return datetime_strftime(value, format_)
 
     def format_time():
         if not value:
@@ -492,7 +505,7 @@ def format_value(field, value, target=None):
         'many2one': format_many2one,
         }
     if isinstance(value, (list, tuple)):
-        return ';'.join(format_value(field, x) for x in value)
+        return ';'.join(format_value(field, x, context=context) for x in value)
     return quote(converts.get(field['type'],
             lambda: value if value is not None else '')())
 
@@ -586,9 +599,10 @@ def test_format_datetime():
         'format': '"%H:%M:%S"',
         }
     for value, result in (
-            (datetime.date(2002, 12, 4), '12/04/2002'),
-            (datetime.datetime(2002, 12, 4), '12/04/2002'),
-            (datetime.datetime(2002, 12, 4, 12, 30), '"12/04/2002 12:30:00"'),
+            (datetime.date(2002, 12, 4), '12/04/02'),
+            (datetime.datetime(2002, 12, 4), '12/04/02'),
+            (untimezoned_date(datetime.datetime(2002, 12, 4, 12, 30)),
+                '"12/04/02 12:30:00"'),
             (False, ''),
             (None, ''),
             ):
@@ -600,7 +614,7 @@ def test_format_date():
         'type': 'date',
         }
     for value, result in (
-            (datetime.date(2002, 12, 4), '12/04/2002'),
+            (datetime.date(2002, 12, 4), '12/04/02'),
             (False, ''),
             (None, ''),
             ):
@@ -655,7 +669,7 @@ def complete_value(field, value):
 
     def complete_datetime():
         yield datetime.date.today()
-        yield datetime.datetime.now()
+        yield datetime.datetime.utcnow()
 
     def complete_date():
         yield datetime.date.today()
@@ -771,6 +785,8 @@ def test_operatorize():
     a = ('a', 'a', 'a')
     b = ('b', 'b', 'b')
     c = ('c', 'c', 'c')
+    null_ = ('d', None, 'x')
+    double_null_ = ('e', None, None)
     for value, result in (
             (['a'], ['a']),
             (['a', 'or', 'b'], [['OR', 'a', 'b']]),
@@ -796,6 +812,8 @@ def test_operatorize():
             (['a', iter(['b', 'or', 'c'])], ['a', [['OR', 'b', 'c']]]),
             ([a, iter([b, ('or',), c])], [a, [['OR', b, c]]]),
             (['a', iter(['b', 'or'])], ['a', [['OR', 'b']]]),
+            ([null_], [null_]),
+            ([null_, 'or', double_null_], [['OR', null_, double_null_]]),
             ):
         assert rlist(operatorize(iter(value))) == result
 
@@ -803,13 +821,14 @@ def test_operatorize():
 class DomainParser(object):
     "A parser for domain"
 
-    def __init__(self, fields):
+    def __init__(self, fields, context=None):
         self.fields = OrderedDict((name, f)
             for name, f in fields.iteritems()
             if f.get('searchable', True))
         self.strings = dict((f['string'].lower(), f)
             for f in fields.itervalues()
             if f.get('searchable', True))
+        self.context = context
 
     def parse(self, input_):
         "Return domain for the input string"
@@ -891,7 +910,7 @@ class DomainParser(object):
                     operator = '!'
                 else:
                     operator = ''
-            formatted_value = format_value(field, value, target)
+            formatted_value = format_value(field, value, target, self.context)
             if (operator in OPERATORS and
                     field['type'] in ('char', 'text', 'selection')
                     and value == ''):
@@ -1086,20 +1105,21 @@ class DomainParser(object):
                             'datetime', 'date', 'time'):
                         if value and '..' in value:
                             lvalue, rvalue = value.split('..', 1)
-                            lvalue = convert_value(field, lvalue)
-                            rvalue = convert_value(field, rvalue)
+                            lvalue = convert_value(field, lvalue, self.context)
+                            rvalue = convert_value(field, rvalue, self.context)
                             yield iter([
                                     (field_name, '>=', lvalue),
                                     (field_name, '<=', rvalue),
                                     ])
                             continue
                     if isinstance(value, list):
-                        value = [convert_value(field, v) for v in value]
+                        value = [convert_value(field, v, self.context)
+                            for v in value]
                         if field['type'] in ('many2one', 'one2many',
                                 'many2many', 'one2one'):
                             field_name += '.rec_name'
                     else:
-                        value = convert_value(field, value)
+                        value = convert_value(field, value, self.context)
                     if 'like' in operator:
                         value = likify(value)
                     if target:
@@ -1192,7 +1212,7 @@ def test_string():
     assert dom.string([]) == ''
     assert dom.string([('surname', 'ilike', '%Doe%')]) == '"(Sur)Name": Doe'
     assert dom.string([('date', '>=', datetime.date(2012, 10, 24))]) == \
-        'Date: >=10/24/2012'
+        'Date: >=10/24/12'
     assert dom.string([('selection', '=', '')]) == 'Selection: '
     assert dom.string([('selection', '=', None)]) == 'Selection: '
     assert dom.string([('selection', '!=', '')]) == 'Selection: !""'
